package jp.co.sds_corp.tool.check;

import java.applet.Applet;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Desktop;
import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.File;
import java.io.FileWriter;
import java.net.URI;
import java.net.URLEncoder;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import javax.swing.Box;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.SwingUtilities;
import javax.swing.filechooser.FileNameExtensionFilter;

import jp.co.sds_corp.util.diag.FileDiagnostics;
import jp.co.sds_corp.util.diag.MemoryDiagnostics;
import jp.co.sds_corp.util.diag.SystemDiagnostics;

public class ClientChecker extends Applet
{
	private static final long serialVersionUID = 1L;
	Frame owner;
	JScrollPane sp;
	JTextArea text;
	JButton btnUpdate;
	JButton btnSave;
	JButton btnSend;
	JCheckBox chkLog;
	FileChooser chooser = new FileChooser();
	String info = "";
	String error = "";
   	SystemDiagnostics sysDiag = new SystemDiagnostics();
	FileDiagnostics fileDiag = new FileDiagnostics();
	MemoryDiagnostics memoryDiag = new MemoryDiagnostics();
	BrowserChecker browserChecker;
	JavaChecker javaChecker;
	NetworkChecker netChecker;
	ProgressDialog progress;

	@Override
    public void init() {
		browserChecker = new BrowserChecker(this);
		javaChecker = new JavaChecker();
		netChecker = new NetworkChecker(this);

		int count = 1;
		count += browserChecker.getCheckCount();
		count += javaChecker.getCheckCount();
		count += netChecker.getCheckCount();
       	owner = getOwnerFrame(this);
		progress = new ProgressDialog(owner);
		progress.setCheckCount(count);
		browserChecker.addProgressListener(progress);
		javaChecker.addProgressListener(progress);
		netChecker.addProgressListener(progress);
	}

    private Frame getOwnerFrame(Component component) {
    	while ((component != null) && !(component instanceof Frame)) {
    		component = component.getParent();
    	}
    	return (Frame )component;
    }

	@Override
    public void start() {
		createLayout();
		update();
	}

	void createLayout() {
		setLayout(new BorderLayout());
		text = new JTextArea();
		text.setEditable(false);
		sp = new JScrollPane(text);
		sp.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_ALWAYS);
		sp.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		add(sp, BorderLayout.CENTER);
		
		JPanel btnPanel = new JPanel();
		btnUpdate = new JButton("最新の状態に更新");
		btnUpdate.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev) {
				update();
			}
		});
		btnPanel.add(btnUpdate);
		btnSave = new JButton("保存");
		btnSave.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev) {
				save();
			}
		});
		btnPanel.add(Box.createHorizontalStrut(30));
		btnPanel.add(btnSave);
/*
		btnSend = new JButton("mail送信");
		btnSend.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev) {
				sendMail();
			}
		});
		btnPanel.add(btnSend);
		chkLog = new JCheckBox("Javaのログも収集する");
		chkLog.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent ev) {
				if (chkLog.isSelected()) {
					if (!checkJavaLog()) {
						chkLog.setSelected(false);
					}
				}
			}

		});
		btnPanel.add(chkLog);
*/
		add(btnPanel, BorderLayout.SOUTH);
	}

	void update() {
		Thread thr = new Thread(new Checker());
		thr.start();
	}

	class Checker implements Runnable {
		public void run() {
			progress.progressNotify(ProgressListener.Type.HOLD, "環境情報取得");
			progress.clear();
			progress.setVisible(true);
			btnUpdate.setEnabled(false);
			btnSave.setEnabled(false);
			loadInfo();
			progress.progressNotify(ProgressListener.Type.FORWARD, "環境情報取得");
			boolean chk = checkEnv();
			SwingUtilities.invokeLater(new Updator(chk));
		}
	}

	class Updator implements Runnable {
		boolean result;
		Updator(boolean result) {
			this.result = result;
		}
		public void run() {
			text.setText(getInfo());
			text.setCaretPosition(0);
			btnUpdate.setEnabled(true);
			btnSave.setEnabled(true);
			progress.setVisible(false);
			if (!result) {
				String title = Message.getString("title.error");
				JOptionPane.showMessageDialog(ClientChecker.this, getError(), title,
					JOptionPane.ERROR_MESSAGE);
			}
		}
	}

	void save() {
		File file = saveFile();
		if (file == null) {
			return;
		}
		String title = Message.getString("title.info");
		String msg = Message.getString("90001");
		JOptionPane.showMessageDialog(this, msg, title,
				JOptionPane.INFORMATION_MESSAGE);
	}

	class FileChooser extends JFileChooser {
		private static final long serialVersionUID = 1L;
		File curDir = null;

		FileChooser() {
			FileNameExtensionFilter filter = new FileNameExtensionFilter(
					"Text file", "txt");
			setFileFilter(filter);
			if (curDir != null) {
				setCurrentDirectory(curDir);
			}
		}

		@Override
		public void approveSelection() {
			File sel = getSelectedFile();
			File dir = sel.getParentFile();
			curDir = dir;
			String name = sel.getName();
			if (!name.endsWith(".txt")) {
				name += ".txt";
			}
			File file = new File(dir, name);
			setSelectedFile(file);
			if (file.exists()) {
				String title = Message.getString("title.confirm");
				String msg = Message.getString("90002");
				int ret = JOptionPane.showConfirmDialog(ClientChecker.this, msg, title,
						JOptionPane.YES_NO_OPTION);
				if(ret != JOptionPane.YES_OPTION) {
					return;
				}
			}
			super.approveSelection();
		}
	}

	SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMddHHmmss");
	File saveFile() {
		String filename = "check_" + sdf.format(new Date()) + ".txt";
		chooser.setSelectedFile(new File(filename));
		int ret = chooser.showSaveDialog(this);
		if(ret != JFileChooser.APPROVE_OPTION) {
			return null;
		}

		File file = chooser.getSelectedFile();
		try {
			FileWriter out = new FileWriter(file);
			out.write(text.getText());
			out.flush();
			out.close();
		} catch (Exception ex) {
			String title = Message.getString("title.error");
			String msg = Message.formatMessage("90003", ex.getMessage());
			JOptionPane.showMessageDialog(ClientChecker.this, msg, title,
					JOptionPane.ERROR_MESSAGE);
			return null;
		}
		return file;
	}

	void sendMail() {
		Desktop desktop = Desktop.getDesktop();
		String uri = "mailto:?to=example@sds_corp.co.jp&";
		try {
			uri += "body=" + URLEncoder.encode(info, "Shift_JIS");
			desktop.mail(new URI(uri));
		} catch (Exception ex) {
			String title = Message.getString("title.error");
			String msg = Message.getString("90004");
			JOptionPane.showMessageDialog(this, msg, title,
					JOptionPane.ERROR_MESSAGE);
		}
	}

	public synchronized void setInfo(String info) {
		this.info = info;
	}
	public synchronized void addInfo(String info) {
		this.info += info;
	}
	public synchronized String getInfo() {
		return this.info;
	}
	public synchronized void setError(String error) {
		this.error = error;
	}
	public synchronized void addError(String error) {
		this.error += error;
	}
	public synchronized String getError() {
		return this.error;
	}

	void loadInfo() {
		setInfo("ClassLoader=" + getClass().getClassLoader() + "\n");
		List<String> list = new ArrayList<String>();
		list.addAll(sysDiag.getSystemProperties());
		list.addAll(memoryDiag.getMemoryInfo());
		list.addAll(sysDiag.getGraphicsEnvironment());
//		list.addAll(fileDiag.getFilesystems());
		list.add("=== filesystems ===");
		File javadir = new File(System.getProperty("java.home"));
		list.add(fileDiag.getFilesystem(javadir));
		for (String msg : list) {
			addInfo(msg + "\n");
		}
	}

	boolean checkEnv() {
		boolean ret = true;
		boolean res;
		setError("");
		addInfo("=== check environment ===\n");
		res = browserChecker.check();
		addInfo(browserChecker.getInfo());
		if (!res) {
			ret = false;
		}
		addError(browserChecker.getError());
		addInfo(browserChecker.getDescription());

		if (!javaChecker.check()) {
			ret = false;
		}
		addError(javaChecker.getError());
		addInfo(javaChecker.getDescription());

		addInfo("=== check network ===\n");
		res = netChecker.check();
		addInfo(netChecker.getInfo());
		if (!res) {
			ret = false;
		}
		addError(netChecker.getError());
		addInfo(netChecker.getDescription());
		return ret;
	}

	boolean checkJavaLog() {
		boolean ret = true;
		String var = System.getProperty("deployment.trace");
		if (var == null || !var.equals("true")) {
			ret = false;
			String title = Message.getString("title.error");
			String msg = Message.getString("90005");
			JOptionPane.showMessageDialog(this, msg, title,
					JOptionPane.ERROR_MESSAGE);
		}
		return ret;
	}

}
