package jp.co.sds_corp.tool.check;

import java.text.MessageFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class Message
{
	static boolean initialized = false;
	static ResourceBundle prop;
	String msgId;
	Level level;
	String title;
	String format;
	String message;

	public enum Level {
		ERR("E"), WARN("W"), INFO("I");
		String code;
		Level(String code) {
			this.code = code;
		}
		public String getCode() {
			return this.code;
		}
		static Level codeOf(String var) {
			if (var == null) {
				return null;
			}
			for (Level level : Level.values()) {
				if (level.getCode().equals(var)) {
					return level;
				}
			}
			return null;
		}
	}

	private Message(String msgId) {
		if (!initialized) {
			load();
		}
		this.msgId = msgId;
	}
	
	private static void load() {
		prop = ResourceBundle.getBundle("msg");
		initialized = true;
	}

	public static Message getMessage(String msgId) {
		if (!initialized) {
			load();
		}
		return getMessage(msgId, prop);
	}

	public static Message getMessage(String msgId, ResourceBundle prop) {
		Message msg = new Message(msgId);
		String def;
		try {
			def = prop.getString(msgId);
		} catch (MissingResourceException mre) {
			def = null;
		}
		if (def == null) {
			return null;
		}
		msg.format = def;
		msg.message = def;
		return msg;
	}

	public static String getString(String msgId) {
		Message msg = getMessage(msgId);
		String ret = msg.toString();
		return ret;
	}

	public static String formatMessage(String msgId, Object... params) {
		Message msg = getMessage(msgId);
		msg.message = msg.format(params);
		return msg.message;
	}

	public String format(Object... params) {
		message = MessageFormat.format(format, params);
		return message;
	}

	public String getMessageId() {
		return msgId;
	}
	public String getFormat() {
		return format;
	}

	public String getMessage() {
		return message;
	}

	public String toString() {
		return format;
	}

}
