package	jp.co.sds_corp.tool.dba;

import java.io.PrintWriter;

public class TableJava extends TableGenerator
{
	String util_pkg = null;
	String pkg = null;
	String[] clm = null;
	int	nDbType;

	public TableJava( String file_name, String pkg, String util_pkg,
					String[] clm, int nDbType )
		throws Exception
	{
		super( file_name );
		this.pkg = pkg;
		this.util_pkg = util_pkg;
		this.clm = clm;
		this.nDbType = nDbType;
	}

	public void create() throws Exception {
		readFile();
		create( getClassName() + ".java" );
	}

	private String getClassName() {
		String[] arr = table.split( "_" );
		StringBuffer buf = new StringBuffer();
		for ( int i = 0; i < arr.length; i++ ) {
			buf.append( arr[i].substring( 0, 1 ).toUpperCase() );
			buf.append( arr[i].substring( 1 ));
		}
		return buf.toString();
	}

	protected void printHeader( PrintWriter out ) {
		out.println( "package " + pkg + ";" );
		out.println( "" );
		out.println( "import java.math.*;" );
		out.println( "import java.sql.*;" );
		out.println( "import jp.co.sds_corp.util.log.CmnLogger;");
		out.println( "" );
		out.println( "import\t" + util_pkg + ".common.*;" );
		out.println( "import\t" + util_pkg + ".db.*;" );
		out.println( "" );
		out.println( "/**" );
		out.println( "* " + table + "テーブルのI/Oクラス" );
		out.println( "* @see DbTable" );
		out.println( "*/" );
		out.println( "public class " + getClassName() + " extends DbTable" );
		out.println( "{" );
		out.println( "\tstatic CmnLogger logger = CmnLogger.getInstance();" );
		printConstructor( out );
	}

	protected void printConstructor( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* 新しい " + getClassName() + " オブジェクトを初期化します" );
		out.println( "\t* @see DbTable" );
		out.println( "\t*/" );
		out.println( "\tpublic " + getClassName() + "() {" );
		out.println( "\t\tsuper( \"" + table + "\" );" );
		if ( clm != null ) {
			for ( int i = 0; i < clm.length; i++ ) {
				out.println( "\t\taddAuditColumn( \"" + clm[i] + "\" );" );
			}
		}
		out.println( "\t\tsetInitColumn( true );" );

		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			out.println( "\t\taddColumn( new DbColumn( \"" + col.getName()
					+ "\", null, " + col.getSqlType() + ", "
					+ col.getSize() + ", "
					+ col.getPrecision() + ", "
					+ col.getScale() + ", 0 ));" );
		}
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printColumn( PrintWriter out, ColumnInfo col, int nPos ) {
		out.println( "\t/**" );
		out.println( "\t* " + col.getJName()
						+ "(" + col.getName() + ")のデータを設定します" );
		out.println( "\t* @param\tvar\t\t設定するデータ" );
//		out.println( "\t* @return\tなし" );
		out.println( "\t*/" );
		out.println( "\tpublic void set" + col.getMethodName() + "( "
						+ col.getJavaType() + " var ) {" );
		out.println( "\t\tsetObject( \"" + col.getName() + "\", var );" );
		out.println( "\t}" );
		out.println( "\t/**" );
		out.println( "\t* " + col.getJName()
						+ "(" + col.getName() + ")のデータを取得します" );
		out.println( "\t* @return\t" + col.getName()  + "の内容" );
		out.println( "\t*/" );
		out.println( "\tpublic " + col.getJavaType()
						+ " get" + col.getMethodName() + "() {" );
		out.println( "\t\treturn (" + col.getJavaType()
						+ " )getObject( \"" + col.getName() + "\" );" );
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printFooter( PrintWriter out ) {
		printSelectSql( out );
		printSelectByKeySql( out );
		printInsertSql( out );
		printDeleteSql( out );
		printDebug( out );
		out.println( "}" );
	}

	protected void printSelectSql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table + "テーブルから全カラムを取得するSQL文を作成し、実行します" );
		out.println( "\t* @param\t\twhere where/order by/group by句" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getSelectSql( String where ) {" );
		out.println( "\t\tString  sql =" );
		out.println( "\t\t\t\"select \"" );

		out.print( getAllColumn() );

		out.println( "\t\t\t+ \" from \" + getTableName() + \" ;" );
		out.println( "\t\tif ( where != null ) {" );
		out.println( "\t\t\tsql += where;" );
		out.println( "\t\t}" );
		out.println( "\t\treturn sql;" );
		out.println( "\t}" );
		out.println( "" );
	}

	private String getAllColumn() {
		StringBuffer buf = new StringBuffer();
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			buf.append( "\t\t\t\t+ \"" + col.getName() );
			if ( i < ( vecClm.size() - 1 )) {
				buf.append( ",\"" );
			} else {
				buf.append( "\"" );
			}
			buf.append( "\n" );
		}
		return buf.toString();
	}

	private String getWhere() {
		StringBuffer buf = new StringBuffer();
		int		nKey = 0;
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			if ( !col.isKey()) {
				continue;
			}
			if ( nKey > 0 ) {
				buf.append( "and \"\n" );
			}
			nKey++;
			if ( col.getJavaType().equals( "String" ) ||
				col.getJavaType().equals( "Timestamp" ) ||
				col.getJavaType().equals( "Date" ))
			{
				buf.append( "\t\t\t+ \"" + col.getName() + " = '\" + " );
				buf.append( "getString( \"" + col.getName() + "\" ) + \"' " );
			} else {
				buf.append( "\t\t\t+ \"" + col.getName() + " = \" + " );
				buf.append( "getString( \"" + col.getName() + "\" ) + \" " );
			}
		}
		if ( nKey == 0 ) {
			buf.append( ";\n" );
		} else {
			buf.append( " \";\n" );
		}
		return buf.toString();
	}

	protected void printSelectByKeySql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table
						+ "テーブルから、キーで指定されたレコードの" );
		out.println( "\t* 全カラムを取得するSQL文を実行します" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getSelectByKeySql() {" );
		if ( countKey() > 0 ) {
			out.println( "\t\tString where = \"where \"" );
			out.println( getWhere() );
		} else {
			out.println( "\t\tString where = \"\";" );
		}
		out.println( "\t\tString sql = getSelectSql( where );" );
		out.println( "\t\treturn sql;" );
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printInsertSql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table + "テーブルにオブジェクトの内容をinsertします" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getInsertSql() {" );
		out.println( "\t\tString  sql = \"insert into \" + getTableName() + \" (\"" );

		out.println( getAllColumn() );

		out.println( "\t\t+ \" ) values (\"" );
		out.print( "\t\t\t" + "+ \"" );
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			String var;
			if ( isAudit( col.getName() )) {
				switch ( nDbType ) {
				case TableLayout.DB_ORACLE:
					var = "sysdate";
					break;
				case TableLayout.DB_SYBASE:
				default:
					var = "getdate()";
				}
			} else {
				var = "?";
			}
			if ( i == vecClm.size() - 1 ) {
				out.println( var + ")\";" );
			} else {
				out.print( var + "," );
			}
		}
		out.println( "\t\treturn sql;" );
		out.println( "\t}" );
		out.println( "" );
	}

	private boolean isAudit( String name ) {
		if ( clm == null ) {
			return false;
		}
		for ( int i = 0; i < clm.length; i++ ) {
			if ( clm[i].equals( name )) {
				return true;
			}
		}
		return false;
	}

	protected void printDeleteSql( PrintWriter out ) {
		out.println( "\t/**" );
		out.println( "\t* " + table + "テーブルから、キーで指定されたレコードを削除します" );
		out.println( "\t* @return\tSQL文" );
		out.println( "\t*/" );
		out.println( "\tpublic String getDeleteSql() {" );
		out.println( "\t\tString sql = \"delete from \" + getTableName() + \" \";" );
		if ( countKey() > 0 ) {
			out.println( "\t\tString where = \"where \"" );
			out.println( getWhere() );
			out.println( "\t\treturn sql + where;" );
		} else {
			out.println( "\t\treturn sql;" );
		}
		out.println( "\t}" );
		out.println( "" );
	}

	protected void printDebug( PrintWriter out ) {
		out.println( "\tpublic void printDebug() {" );
		for ( int i = 0; i < vecClm.size(); i++ ) {
			ColumnInfo col = (ColumnInfo )vecClm.get( i );
			out.println( "\t\tlogger.debug( this, \"" + col.getName()
							+ "=[\" + getString( \"" + col.getName()
							+ "\" ) + \"]\" );" );
		}
		out.println( "\t}" );
		out.println( "" );
	}

	public static void main( String[] args ) {
		try {
			String[] audit = { "audit_datetime" };
			for ( int i = 0; i < args.length; i++ ) {
				TableJava tbl = new TableJava( args[i], "focus.table", "util", audit, TableLayout.DB_ORACLE );
				tbl.create();
			}
		} catch ( Exception e ) {
			e.printStackTrace();
		}
	}

}

