package	jp.co.sds_corp.tool.dba;

import java.io.FileWriter;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.ResultSet;
import java.sql.Types;
import java.util.Vector;

import jp.co.sds_corp.util.config.DOMConfiguration;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.db.DbAccess;

public class TextGenerator
{
	DbAccess	db;
	String		schema;

	public TextGenerator() {
	}

	public void init( IConfiguration config ) throws Exception {
		IConfiguration c = config.getChild( "db" );
		db = new DbAccess( c );
		db.connect();

		schema = c.get( "schema" );
	}

	Vector<String> vecKey = new Vector<String>();
	private void getPkey( DatabaseMetaData meta, String table ) throws Exception {
		vecKey.clear();
		ResultSet rset = meta.getPrimaryKeys( null, schema, table );
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			vecKey.add( rset.getString( "COLUMN_NAME" ) );
		}
	}

	public void createText( String table ) throws Exception {
		Connection con = db.getConnection();
		DatabaseMetaData meta = con.getMetaData();
		getPkey( meta, table );

		PrintWriter pw = new PrintWriter( new FileWriter( table + ".txt" ));
		pw.println( "No" + "\t" + "Name" + "\t" + "Column" + "\t" + "Type"
					+ "\t" + "Key" + "\t" + "Description" );
		ResultSet rset = meta.getColumns( null, schema, table, null );
		int no = 1;
		for ( ;; ) {
			if ( !rset.next()) {
				break;
			}
			short sqlt = rset.getShort( "DATA_TYPE" );
			String col = rset.getString( "COLUMN_NAME" );
			String type = rset.getString( "TYPE_NAME" ).toLowerCase();
			int size = rset.getInt( "COLUMN_SIZE" );
			int prec = rset.getInt( "DECIMAL_DIGITS" );
			switch ( sqlt ) {
			case Types.CHAR:
			case Types.VARCHAR:
//				if ( size > 1 ) {
					type = type + "(" + size + ")";
//				}
				break;
			case Types.DECIMAL:
			case Types.NUMERIC:
				type = type + "(" + size;
				if ( prec > 0 ) {
					type = type + "." + prec;
				}
				type = type + ")";
				break;
			}
			String key;
			if ( vecKey.indexOf( col ) >= 0 ) {
				key = "○";
			} else {
				key = "";
			}
			pw.println( no + "\t"
					+ col + "\t"
					+ col + "\t"
					+ type + "\t"
					+ key + "\t"
					+ "" );
			no++;
		}
		pw.close();
	}
/*
	public void createText( String table ) throws Exception {
		PrintWriter pw = new PrintWriter( new FileWriter( table + ".txt" ));
		pw.println( "No" + "\t" + "Name" + "\t" + "Column" + "\t" + "Type"
					+ "\t" + "Key" + "\t" + "Description" );
		DbTable tbl = new DbTable();
		db.executeQuery( "select * from " + table + " where 0=1" );
		db.fetch( tbl );
		db.cancel();

		for ( int i = 0; i < tbl.getColumnCount(); i++ ) {
			DbColumn col = tbl.getColumn( i );
			String type = col.getTypeName();
			switch ( col.getType() ) {
			case Types.CHAR:
			case Types.DECIMAL:
			case Types.NUMERIC:
			case Types.VARCHAR:
//				if ( col.getSize() > 1 ) {
					type = type + "(" + col.getSize() + ")";
//				}
			}
			pw.println( (i+1) + "\t"
					+ col.getLabel() + "\t"
					+ col.getName() + "\t"
					+ type + "\t"
					+ "" + "\t"
					+ "" );
		}
		pw.close();
	}
*/

	public static void main( String[] args ) {
		try {
			String file = System.getProperty( "dba.config" );
			DOMConfiguration config = new DOMConfiguration( file );

			TextGenerator txt = new TextGenerator();
			txt.init( config );
			for ( int i = 0; i < args.length; i++ ) {
				txt.createText( args[i] );
			}
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}
}
