package	jp.co.sds_corp.tool.http;

import	java.net.CookieHandler;
import	java.net.CookieManager;
import	java.net.CookiePolicy;
import	java.util.ArrayList;
import	java.util.List;
import	java.util.HashMap;
import	java.util.Map;

import	jp.co.sds_corp.util.config.DOMConfiguration;
import	jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import	jp.co.sds_corp.util.log.CmnLogger;
import	jp.co.sds_corp.tool.test.StressExecutor;
import	jp.co.sds_corp.tool.test.StressTester;

public class StressWeb implements StressExecutor<RequestInfo>
{
	CmnLogger logger = CmnLogFactory.getInstance().getLogger(this);
	StressTester<RequestInfo>	stress;
	HttpRequester requester = new HttpRequester();
	long	interval = 1000;
	int		count = 10;
	int		threadNum = 1;
	List<RequestInfo> listLogin = new ArrayList<RequestInfo>();
	List<RequestInfo> listTarget = new ArrayList<RequestInfo>();
	Map<String,String> mapResult = new HashMap<String,String>();
	CookieManager manager;
	boolean singleSession = false;
	CookiePolicy policy = CookiePolicy.ACCEPT_ALL;

	public void init() throws Exception {
		IConfiguration doc = DOMConfiguration.findResource( "stressweb.xml" );
		IConfiguration config = doc.getChild( "StressWeb" );
		init( config );
	}

	public void init( IConfiguration config ) throws Exception {
		String var;
		var = config.getAttributes().get( "interval", null );
		if ( var != null && var.length() > 0 ) {
			interval = Long.valueOf( var ).longValue();
		}
		var = config.getAttributes().get( "count", null );
		if ( var != null && var.length() > 0 ) {
			count = Integer.valueOf( var ).intValue();
		}
		var = config.getAttributes().get( "thread", null );
		if ( var != null && var.length() > 0 ) {
			threadNum = Integer.valueOf( var ).intValue();
		}
		var = config.getAttributes().get( "singleSession", null );
		if ( var != null && var.length() > 0 ) {
			singleSession = Boolean.valueOf( var ).booleanValue();
		}
		var = config.getAttributes().get( "cookie", null );
		if ( var != null && var.length() > 0 ) {
			if ( var.equals( "ACCEPT_ALL" )) {
				policy = CookiePolicy.ACCEPT_ALL;
			} else
			if ( var.equals( "ACCEPT_NONE" )) {
				policy = CookiePolicy.ACCEPT_NONE;
			} else
			if ( var.equals( "ACCEPT_ORIGINAL_SERVER" )) {
				policy = CookiePolicy.ACCEPT_ORIGINAL_SERVER;
			}
		}

		List<IConfiguration> list;
		list = config.listChild( "target" );
		for ( IConfiguration c : list ) {
			RequestInfo info = RequestInfo.createInstance( c );
			if ( info != null ) {
				listTarget.add( info );
			}
		}
		list = config.listChild( "login" );
		for ( IConfiguration c : list ) {
			RequestInfo info = RequestInfo.createInstance( c );
			if ( info != null ) {
				listLogin.add( info );
			}
		}

		stress = new StressTester( this );
		stress.setPoolSize( threadNum );

		manager = new ThreadableCookieManager( singleSession, policy );
		CookieHandler.setDefault( manager );
	}

	public void login() throws Exception {
		login( requester );
	}

	public void login( HttpRequester requester ) throws Exception {
		for ( RequestInfo info : listLogin ) {
			logger.info( "login URL=" + info.getUrl() );
			requester.request( info );
			mapResult.putAll( requester.getResult() );
		}
	}

	Map<Long,HttpRequester> mapReq = new HashMap<Long,HttpRequester>();

	@Override
	public void startThread() throws Exception {
		HttpRequester requester = new HttpRequester();
		if ( singleSession ) {
			requester.setResult( mapResult );
		} else {
			login( requester );
		}
		long id = Thread.currentThread().getId();
		mapReq.put( id, requester );
	}

	@Override
	public void test( RequestInfo info ) throws Exception {
		long id = Thread.currentThread().getId();
		HttpRequester requester = mapReq.get( id );
		requester.request( info );
	}

	@Override
	public void stopThread() throws Exception {
		long id = Thread.currentThread().getId();
		HttpRequester requester = mapReq.get( id );
		requester.close();
	}

	public void start() throws Exception {
		stress.test( listTarget, interval, count );
		stress.stop();
	}

	public static void main( String[] args ) {
		StressWeb obj = new StressWeb();
		try {
			obj.init();
			if ( obj.singleSession ) {
				obj.login();
			}
			obj.start();
		} catch ( Exception ex ) {
			ex.printStackTrace();
		}
	}

}
