package	jp.co.sds_corp.tool.http;

import	java.io.IOException;
import	java.net.CookieManager;
import	java.net.CookiePolicy;
import	java.net.CookieStore;
import	java.net.URI;
import	java.util.HashMap;
import	java.util.List;
import	java.util.Map;

public class ThreadableCookieManager extends CookieManager
{
	boolean singleThread;
	CookiePolicy policy;
	Map<Long,CookieManager> mapManager = new HashMap<Long,CookieManager>();

	public ThreadableCookieManager( boolean singleThread, CookiePolicy policy ) {
		super();
		this.singleThread = singleThread;
		this.policy = policy;
		setCookiePolicy( policy );
	}

	CookieManager getManager() {
		if ( singleThread ) {
			return this;
		}
		long id = Thread.currentThread().getId();
		CookieManager ret = mapManager.get( id );
		if ( ret == null ) {
			ret = new CookieManager();
			ret.setCookiePolicy( policy );
			mapManager.put( id, ret );
		}
		return ret;
	}

	@Override
	public CookieStore getCookieStore() {
		if ( singleThread ) {
			return super.getCookieStore();
		}
		CookieManager manager = getManager();
		return manager.getCookieStore();
	}

	@Override
	public Map<String,List<String>> get(URI uri, Map<String,List<String>> requestHeaders) 
		throws IOException
	{
		if ( singleThread ) {
			return super.get( uri, requestHeaders );
		}
		CookieManager manager = getManager();
		return manager.get( uri, requestHeaders );
	}

	@Override
	public void put(URI uri, Map<String,List<String>> responseHeaders)
		throws IOException
	{
		if ( singleThread ) {
			super.put( uri, responseHeaders );
			return;
		}
		CookieManager manager = getManager();
		manager.put( uri, responseHeaders );
	}

}
